﻿using System;
using System.IO.IsolatedStorage;
using System.Diagnostics;
using System.Collections.Generic;

namespace SGPM.PhonePER
{
    /// <summary>
    /// Clase que sirve de soporte para almacenar las preferencias del usuario perito
    /// </summary>
    public class AppSettings
    {
        // Almacenamiento de la configuración en el isolated storage 
        IsolatedStorageSettings settings;

        // Los nombres de las claves del isolated de la configuración
        const string UsernameSettingKeyName = "UsernameSetting";
        const string PasswordSettingKeyName = "PasswordSetting";
        const string RememberMeSettingKeyName = "RememberMeSetting";

        // Valores por defecto de la configuración
        const string UsernameSettingDefault = "";
        const string PasswordSettingDefault = "";
        const bool RememberMeSettingDefault = false;

        /// <summary>
        /// Constructor que recuperar la configuración de la aplicación
        /// </summary>
        public AppSettings()
        {
            // Recupera la configuración de la aplicación.
            try
            {
                settings = IsolatedStorageSettings.ApplicationSettings;
            }
            catch (System.IO.IsolatedStorage.IsolatedStorageException ex)
            {
                // handle exception
                System.Diagnostics.Debug.WriteLine(ex);
            }
        }

        /// <summary>
        /// Actualiza el valor de un elemento. Si el elemento no existe, entonces lo añade
        /// </summary>
        /// <param name="Key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool AddOrUpdateValue(string Key, Object value)
        {
            bool valueChanged = false;

            // Si el elemento existe
            if (settings.Contains(Key))
            {
                // Si el valor ha cambiado
                if (settings[Key] != value)
                {
                    // Almacena el nuevo valor
                    settings[Key] = value;
                    valueChanged = true;
                }
            }
            // En otro caso, crea la clave.
            else
            {
                settings.Add(Key, value);
                valueChanged = true;
            }
            return valueChanged;
        }

        /// <summary>
        /// Recupera el valor actual de un elemento, o si este no se encuentra, 
        /// lo actualiza con el valor por defecto
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="Key"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public T GetValueOrDefault<T>(string Key, T defaultValue)
        {
            T value;

            // Si la clave existe, recupera el valor
            if (settings.Contains(Key))
            {
                value = (T)settings[Key];
            }
            // En otro caso, devuelve el valor por defecto
            else
            {
                value = defaultValue;
            }
            return value;
        }

        /// <summary>
        /// Almacena la configuración
        /// </summary>
        public void Save()
        {
            settings.Save();
        }

        /// <summary>
        /// Propiedad get y set para almacenar la configuración del nombre de usuario
        /// </summary>
        public string UsernameSetting
        {
            get
            {
                if (RememberMeSetting)
                {
                    return GetValueOrDefault<string>(UsernameSettingKeyName, UsernameSettingDefault);
                }
                else
                {
                    return UsernameSettingDefault;
                }
            }
            set
            {
                if (AddOrUpdateValue(UsernameSettingKeyName, value))
                {
                    Save();
                }
            }
        }

        /// <summary>
        /// Propiedad get y set para almacenar la configuración de la password
        /// </summary>
        public string PasswordSetting
        {
            get
            {
                if (RememberMeSetting)
                {
                    return GetValueOrDefault<string>(PasswordSettingKeyName, PasswordSettingDefault);
                }
                else
                {
                    return PasswordSettingDefault;
                }
            }
            set
            {
                if (AddOrUpdateValue(PasswordSettingKeyName, value))
                {
                    Save();
                }
            }
        }

        /// <summary>
        /// Propiedad get y set para almacenar la configuración del Recordar password
        /// </summary>
        public bool RememberMeSetting
        {
            get
            {
                return GetValueOrDefault<bool>(RememberMeSettingKeyName, RememberMeSettingDefault);
            }
            set
            {
                if (AddOrUpdateValue(RememberMeSettingKeyName, value))
                {
                    Save();
                }
            }
        }
    }
}
