﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con los clientes
    /// </summary>
    public class ClienteController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Cliente/
        /// <summary>
        /// Listado de clientes
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            return View();

        }

        /// <summary>
        /// Solicita un listado de clientes 
        /// </summary>
        /// <param name="nombre">Nombre por el que filtrar.</param>
        /// <param name="baja">The destination account.</param>
        /// <param name="jtStartIndex">Indice de los registros a partir del que hay que listar.</param>
        /// <param name="jtPageSize">Tamaño de la página.</param>
        /// <param name="jtSorting">Ordenación del listado.</param>
        /// <returns>Un objeto Json con el listado de clientes.</returns>
        [HttpPost]
        public JsonResult ClienteList(string nombre = null, int baja = (int)FiltroEstadoBaja.Todos, int jtStartIndex = 0, int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var clientesCount = _ws.GetClientesCountByFilter(nombre, baja);

                    if (clientesCount < 0)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                    }

                    var clientes = _ws.ListClientes(this._gestor, nombre, baja, jtStartIndex, jtPageSize, jtSorting);

                    return Json(new { Result = "OK", Records = clientes, TotalRecordCount = clientesCount });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Inserta o actualiza un cliente
        /// </summary>
        /// <param name="cliente">Cliente a insertar o actualizar.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult ClienteUpdate(Cliente cliente)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    if (cliente == null)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error. No se encuentran los datos." });
                    }

                    //Si el item tiene fecha de baja, no se puede modificar
                    if (!string.IsNullOrEmpty(cliente.FechaBaja.ToString()))
                    {
                        return Json(new { Result = "ERROR", Message = "El registro ya se encuentra de baja." });
                    }

                    var addedCliente = _ws.UpdateCliente(this._gestor, cliente);
                    return Json(new { Result = "OK", Record = addedCliente });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Realiza la baja lógica de un cliente
        /// </summary>
        /// <param name="idCliente">Id. del cliente a dar de baja.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult ClienteDelete(int idCliente)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    bool bOk = _ws.DeleteCliente(this._gestor, idCliente);

                    if (bOk)
                    {
                        return Json(new { Result = "OK" });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error al intentar borrar el elemento." });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        #endregion

    }
}
