﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

using System.Net;
using System.IO;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con las órdenes de trabajo
    /// </summary>
    public class OrdenController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Orden/
        /// <summary>
        /// Listado de órdenes
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
            {
                // Carga de los Filtros de los desplegables

                try
                {

                    // Clientes
                    List<SelectListItem> clientes = new List<SelectListItem>();
                    clientes.Add(new SelectListItem { Text = "- Elija cliente -", Value = "0" });
                    clientes.AddRange(_ws.ListAllClientes(this._gestor).Select(c => new SelectListItem
                    {
                        Text = String.Format("{0} {1}", c.Nombre, c.Apellidos),
                        Value = c.IdCliente.ToString()
                    }).ToList());
                    ViewBag.Clientes = clientes;

                    // Peritos
                    var peritos = _ws.ListAllUsuarios(this._gestor).Where(c => c.Tipo == (int)TipoUsuario.Perito).Select(c => new SelectListItem
                    {
                        Text = String.Format("{0} {1}", c.Nombre, c.Apellidos),
                        Value = c.IdUsuario.ToString()
                    }).ToList();
                    peritos.Insert(0, new SelectListItem { Text = "- Elija un perito -", Value = "0" });
                    ViewBag.Peritos = peritos;

                    // Estados
                    var estados = _ws.ListEstadosOrden(this._gestor).Select(c => new SelectListItem
                    {
                        Text = c.Nombre,
                        Value = c.IdEstado.ToString()
                    }).ToList();
                    estados.Insert(0, new SelectListItem { Text = "- Elija un estado -", Value = "0" });
                    ViewBag.Estados = estados;

                    return View();
                }
                catch (Exception ex)
                {
                    Tracer.Logger.Error(ex);
                    // La sesión ha expirado
                    return RedirectToAction("LogOn", "Account");
                }
            }
        }


        /// <summary>
        /// Solicita un listado de órdenes de trabajo
        /// </summary>
        /// <param name="idCliente">Cliente del que se quieren listas las órdenes de trabajo.</param>
        /// <param name="idPerito">Perito del que se desean obtener las ordenes</param>
        /// <param name="idEstado">Filtro por estado de la orden</param>
        /// <param name="jtStartIndex">Indice de los registros a partir del que hay que listar.</param>
        /// <param name="jtPageSize">Tamaño de la página.</param>
        /// <param name="jtSorting">Ordenación del listado.</param>
        /// <returns>Un objeto Json con el listado de órdenes de trabajo.</returns>
        [HttpPost]
        public JsonResult OrdenList(int idCliente = 0, int idPerito = 0, int idEstado = (int)EstadosOrden.Todos, int jtStartIndex = 0,
            int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var ordenesCount = _ws.GetOrdenesCountByFilter(idCliente, idPerito, idEstado);

                    if (ordenesCount < 0)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                    }

                    var ordenes = _ws.ListOrdenes(this._gestor, idCliente, idPerito, idEstado, jtStartIndex, jtPageSize, jtSorting);

                    return Json(new { Result = "OK", Records = ordenes, TotalRecordCount = ordenesCount });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Inserta o actualiza una orden de trabajo
        /// </summary>
        /// <param name="orden">Orden de trabajo a insertar o actualizar.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult OrdenUpdate(OrdenTrabajo orden)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    if (orden == null)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error. No se encuentran los datos." });
                    }

                    //Si la orden de trabajo se encuentra en estado "Cerrada", no se puede modificar
                    if (orden.IdEstado == (int)EstadosOrden.Cerrada)
                    {
                        return Json(new { Result = "ERROR", Message = "La orden de trabajo se ecuentra 'Cerrada', no se puede modificar." });
                    }

                    var addedOrden = _ws.UpdateOrden(this._gestor, orden);
                    if (addedOrden != null)
                    {
                        return Json(new { Result = "OK", Record = addedOrden });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Error al insertar/actualizar el registro" });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Realiza la baja de una orden de trabajo
        /// </summary>
        /// <param name="idOrdenTrabajo">Id. de la orden de trabajo a dar de baja.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult OrdenDelete(int idOrdenTrabajo)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    bool bOk = _ws.DeleteOrden(this._gestor, idOrdenTrabajo);

                    if (bOk)
                    {
                        return Json(new { Result = "OK" });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error al intentar borrar el elemento." });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita la imagen asociada a una orden
        /// </summary>
        /// <param name="idOrdenTrabajo">Id. de la orden de trabajo a recuperar la foto.</param>
        /// <returns>Un array de bytes con la foto asociada.</returns>
        public ActionResult GetFoto(int idOrdenTrabajo)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var foto = _ws.GetFoto(this._gestor, idOrdenTrabajo);

                    return File(foto, "image/jpeg");
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        #endregion

    }
}
