﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con las pólizas
    /// </summary>
    public class PolizaController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Poliza/
        /// <summary>
        /// Listado de propiedades
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
            {

                var clientes = _ws.ListAllClientes(this._gestor).Select(c => new SelectListItem
                {
                    Text = String.Format("{0} {1}",c.Nombre ,c.Apellidos),
                    Value = c.IdCliente.ToString()
                }).ToList();

                ViewBag.Clientes = clientes;
                return View();
            }
        }


        /// <summary>
        /// Solicita un listado de pólizas
        /// </summary>
        /// <param name="idCliente">Cliente del que se quieren listas las pólizas.</param>
        /// <param name="idPropiedad">Propiedad del que se quieren listas las pólizas.</param>
        /// <param name="baja">Indica si queremos consultar las pólizas activas o las de baja.</param>
        /// <param name="jtStartIndex">Indice de los registros a partir del que hay que listar.</param>
        /// <param name="jtPageSize">Tamaño de la página.</param>
        /// <param name="jtSorting">Ordenación del listado.</param>
        /// <returns>Un objeto Json con el listado de pólizas.</returns>
        [HttpPost]
        public JsonResult PolizaList(int idCliente, int idPropiedad = 0, int baja = (int)FiltroEstadoBaja.Todos, int jtStartIndex = 0,
            int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var polizasCount = _ws.GetPolizasCountByFilter(idCliente, idPropiedad, baja);

                    if (polizasCount < 0)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                    }

                    var polizas = _ws.ListPolizas(this._gestor, idCliente, idPropiedad, baja, jtStartIndex, jtPageSize, jtSorting);

                    return Json(new { Result = "OK", Records = polizas, TotalRecordCount = polizasCount });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Inserta o actualiza una poliza
        /// </summary>
        /// <param name="propiedad">Póliza a insertar o actualizar.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult PolizaUpdate(Poliza poliza)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    if (poliza == null)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error. No se encuentran los datos." });
                    }

                    //Si el item tiene fecha de baja, no se puede modificar
                    if (!string.IsNullOrEmpty(poliza.FechaBaja.ToString()))
                    {
                        return Json(new { Result = "ERROR", Message = "El registro ya se encuentra de baja." });
                    }

                    var addedPoliza = _ws.UpdatePoliza(this._gestor, poliza);
                    if (addedPoliza != null)
                    {
                        return Json(new { Result = "OK", Record = addedPoliza });
                    }
                    else
                    {
                        return Json(new { Result = "OK", Message = "Error al insertar/actualizar el registro" });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Realiza la baja lógica de una póliza
        /// </summary>
        /// <param name="idPropiedad">Id. de la póliza a dar de baja.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult PolizaDelete(int idPoliza)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    bool bOk = _ws.DeletePoliza(this._gestor, idPoliza);

                    if (bOk)
                    {
                        return Json(new { Result = "OK" });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error al intentar borrar el elemento." });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        #endregion

    }
}
