﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con las propiedades
    /// </summary>
    public class PropiedadController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Propiedad/
        /// <summary>
        /// Listado de propiedades
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
            {

                var clientes = _ws.ListAllClientes(this._gestor).Select(c => new SelectListItem { 
                    Text = c.Nombre + " " + c.Apellidos, Value = c.IdCliente.ToString() }).ToList();

                ViewBag.Clientes = clientes;
                return View();
            }
        }

        /// <summary>
        /// Solicita un listado de propiedades
        /// </summary>
        /// <param name="idCliente">Cliente del que se quieren listas las propiedades.</param>
        /// <param name="baja">Indica si queremos consultar las propiedades activas o las de baja.</param>
        /// <param name="jtStartIndex">Indice de los registros a partir del que hay que listar.</param>
        /// <param name="jtPageSize">Tamaño de la página.</param>
        /// <param name="jtSorting">Ordenación del listado.</param>
        /// <returns>Un objeto Json con el listado de propiedades.</returns>
        [HttpPost]
        public JsonResult PropiedadList(int idCliente, int baja = (int)FiltroEstadoBaja.Todos, int jtStartIndex = 0,
            int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var propiedadesCount = _ws.GetPropiedadesCountByFilter(idCliente, baja);

                    if (propiedadesCount < 0)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                    }

                    var propiedades = _ws.ListPropiedades(this._gestor, idCliente, baja, jtStartIndex, jtPageSize, jtSorting);

                    return Json(new { Result = "OK", Records = propiedades, TotalRecordCount = propiedadesCount });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Inserta o actualiza un propiedad
        /// </summary>
        /// <param name="propiedad">Propiedad a insertar o actualizar.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult PropiedadUpdate(Propiedad propiedad)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    if (propiedad == null)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error. No se encuentran los datos." });
                    }

                    //Si el item tiene fecha de baja, no se puede modificar
                    if (!string.IsNullOrEmpty(propiedad.FechaBaja.ToString()))
                    {
                        return Json(new { Result = "ERROR", Message = "El registro ya se encuentra de baja." });
                    }

                    var addedPropiedad = _ws.UpdatePropiedad(this._gestor, propiedad);
                    if (addedPropiedad != null)
                    {
                        return Json(new { Result = "OK", Record = addedPropiedad });
                    }
                    else
                    {
                        return Json(new { Result = "OK", Message = "Error al insertar/actualizar el registro"});
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Realiza la baja lógica de un propiedad
        /// </summary>
        /// <param name="idPropiedad">Id. del propiedad a dar de baja.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        public JsonResult PropiedadDelete(int idPropiedad)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    bool bOk = _ws.DeletePropiedad(this._gestor, idPropiedad);

                    if (bOk)
                    {
                        return Json(new { Result = "OK" });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error al intentar borrar el elemento." });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        #endregion

    }
}
