﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.ProxyWSWebPER; 

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador representa la clase base para todos los controladores de la aplicación.
    /// Provee de acceso al usuario gestor que se ha logeado a la aplicación y de una serie de métodos comunes para la carga
    /// de datos en los formularios
    /// </summary>
    public abstract class RepositoryBasedController : Controller
    {

        #region Propiedades Private/protected
        
        //
        // Recuperación del usuario logeado
        protected Usuario _gestor = System.Web.HttpContext.Current.Session["Usuario"] as Usuario;

        #endregion

        #region Métodos comunes

        /// <summary>
        /// Solicita un listado de clientes para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de clientes.</returns>
        [HttpPost]
        public virtual JsonResult GetClienteOptions()
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var clientes = _ws.ListAllClientes(this._gestor).Select(c => new
                    {
                        DisplayText = String.Format("{0} {1}", c.Nombre, c.Apellidos),
                        Value = c.IdCliente
                    }).OrderBy(x => x.DisplayText).ToList();

                    clientes.Insert(0, new { DisplayText = "- Elija un cliente -", Value = 0 });

                    return Json(new { Result = "OK", Options = clientes });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita un listado de peritos para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de peritos.</returns>
        [HttpPost]
        public virtual JsonResult GetPeritosOptions()
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var peritos = _ws.ListAllUsuarios(this._gestor).Where(c => c.Tipo == (int)TipoUsuario.Perito).Select(c => new
                    {
                        DisplayText = String.Format("{0} {1}",c.Nombre ,c.Apellidos),
                        Value = c.IdUsuario
                    }).OrderBy(c => c.DisplayText).ToList();

                    peritos.Insert(0, new { DisplayText = "- Elija un perito -", Value = 0 });

                    return Json(new { Result = "OK", Options = peritos });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita un listado de propiedades de un cliente para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de propiedades de un cliente.</returns>
        [HttpPost]
        public virtual JsonResult GetPropiedadOptions(int idCliente)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var propiedades = _ws.ListPropiedadesByCliente(this._gestor, idCliente).Select(c => new
                    {
                        DisplayText = String.Format("{0}, {1}, {2}, {3}, {4}, {5}",  c.Calle, c.Numero, c.Piso,
                        c.Letra,
                        c.CP,
                        c.Localidad),
                        Value = c.IdPropiedad
                    });


                    return Json(new { Result = "OK", Options = propiedades });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita un listado de pólizas de un cliente para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de órdenes de trabajo de un cliente.</returns>
        [HttpPost]
        public virtual JsonResult GetPolizaOptions(int idCliente)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var ordenes = _ws.ListPolizasByCliente(this._gestor, idCliente).Select(c => new
                    {
                        DisplayText =String.Format("{0} - {1}, {2}, {3}, {4}, {5}",   c.NPoliza, c.Propiedad.Calle, 
                            c.Propiedad.Numero, c.Propiedad.Piso, 
                            c.Propiedad.Letra, c.Propiedad.Localidad),
                        Value = c.IdPoliza
                    });


                    return Json(new { Result = "OK", Options = ordenes });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita un listado de tipos de siniestro para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de tipos de siniestro.</returns>
        [HttpPost]
        public virtual JsonResult GetTipoSiniestroOptions()
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var ordenes = _ws.ListTiposSiniestro(this._gestor).Select(c => new
                    {
                        DisplayText = c.Nombre,
                        Value = c.IdTipoSiniestro
                    });


                    return Json(new { Result = "OK", Options = ordenes });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Solicita un listado de provincias para mostrar en un desplegable
        /// </summary>
        /// <returns>Un objeto Json con el listado de provincias.</returns>
        [HttpPost]
        public virtual JsonResult GetProvinciaOptions()
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var provincias = _ws.ListProvincias(this._gestor).Select(c => new
                    {
                        DisplayText = c.Nombre,
                        Value = c.IdProvincia
                    });
                    return Json(new { Result = "OK", Options = provincias });
                }
            }
            catch (Exception ex)
            {
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        #endregion
    }
}
