﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con los usuarios
    /// </summary>
    public class UsuarioController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Usuario/
        /// <summary>
        /// Listado de usuarios
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            return View();

        }

        /// <summary>
        /// Solicita un listado de usuarios 
        /// </summary>
        /// <param name="nombre">Nombre por el que filtrar.</param>
        /// <param name="baja">Filtro para obtener los usuarios de alta, de baja o todos.</param>
        /// <param name="jtStartIndex">Indice de los registros a partir del que hay que listar.</param>
        /// <param name="jtPageSize">Tamaño de la página.</param>
        /// <param name="jtSorting">Ordenación del listado.</param>
        /// <returns>Un objeto Json con el listado de usuarios.</returns>
        [HttpPost]
        public JsonResult UsuarioList(string nombre = null, int tipo = (int)TipoUsuario.Todos, int baja = (int)FiltroEstadoBaja.Todos, 
            int jtStartIndex = 0, int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    var usuariosCount = _ws.GetUsuariosCountByFilter(nombre, tipo, baja);

                    if (usuariosCount < 0)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                    }

                    var usuarios = _ws.ListUsuarios(this._gestor, nombre, tipo, baja, jtStartIndex, jtPageSize, jtSorting);

                    return Json(new { Result = "OK", Records = usuarios, TotalRecordCount = usuariosCount });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Inserta o actualiza un usuario
        /// </summary>
        /// <param name="usuario">Usuario a insertar o actualizar.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        [SessionExpireFilter]
        public JsonResult UsuarioUpdate(Usuario usuario)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    if (usuario == null)
                    {
                        //log
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error. No se encuentran los datos." });
                    }

                    //Si el item tiene fecha de baja, no se puede modificar
                    if (!string.IsNullOrEmpty(usuario.FechaBaja.ToString()))
                    {
                        return Json(new { Result = "ERROR", Message = "El registro ya se encuentra de baja." });
                    }

                    // Si es un alta hay que comprobar si el login ya existe
                    if (usuario.IdUsuario == 0)
                    {
                        if (!_ws.IsLoginFree(usuario.Login))
                        {
                            throw new Exception(string.Format("El login {0} ya existe", usuario.Login));
                        }

                    }

                    var addedUsuario = _ws.UpdateUsuario(this._gestor, usuario);
                    return Json(new { Result = "OK", Record = addedUsuario });
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        /// <summary>
        /// Realiza la baja lógica de un usuario
        /// </summary>
        /// <param name="idUsuario">Id. del usuario a dar de baja.</param>
        /// <returns>Un objeto Json con el resultado de la operación.</returns>
        [HttpPost]
        [SessionExpireFilter]
        public JsonResult UsuarioDelete(int idUsuario)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    if (!ModelState.IsValid)
                    {
                        return Json(new { Result = "ERROR", Message = "¡El formulario no es válido! Por favor, revise los datos." });
                    }

                    bool bOk = _ws.DeleteUsuario(this._gestor, idUsuario);

                    if (bOk)
                    {
                        return Json(new { Result = "OK" });
                    }
                    else
                    {
                        return Json(new { Result = "ERROR", Message = "Se ha producido un error al intentar borrar el elemento. Compruebe que no sea un perito con ordenes descargadas." });
                    }
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }

        #endregion

    }
}
