﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using SGPM.WebServices.ModeloDatos;
using SGPM.WebServices;

using System.IO;

namespace SGPM.WebServices
{
    /// <summary>
    /// Interfaz del servicio web WSWebPer destinada a la aplicación de administración 
    /// </summary>
    [ServiceContract]
    public interface IWSWebPER
    {

        #region Provincias

        /// <summary>
        /// Devuelve un listado con las provincias del sistema
        /// </summary>
        /// <returns>Un listado con todas las provincias</returns>
        [OperationContract]
        List<Provincia> ListProvincias(Usuario gestor);

        #endregion

        #region Tipos de siniestro
        /// <summary>
        /// Devuelve un listado con los tipos de siniestro del sistema
        /// </summary>
        /// <returns>Un listado con todos los tipos de siniestro</returns>
        [OperationContract]
        List<TipoSiniestro> ListTiposSiniestro(Usuario gestor);

        #endregion

        #region Estados de la orden de trabajo
        /// <summary>
        /// Devuelve un listado con los estados de la orden de trabajo
        /// </summary>
        /// <returns>Un listado con todos los estados de la orden de trabajo</returns>
        [OperationContract]
        List<EstadoOrden> ListEstadosOrden(Usuario gestor);

        #endregion


        #region Usuarios
        /// <summary>
        /// Realiza el control de acceso a la aplicación de un usuario Gestor
        /// </summary>
        /// <param name="login">Nombre de usuario del gestor</param>
        /// <param name="password">Contraseña del gestor</param>
        /// <returns>El gestor que ha accedido correctamente a la aplicación</returns>
        [OperationContract]
        Usuario ValidaUsuario(string login, string password);

        /// <summary>
        /// Realiza el cambio de contraseña de un usuario Gestor
        /// </summary>
        /// <param name="gestor">Usuario gestor del que se quiere cambiar la constraseña</param>
        /// <param name="password">La contraseña a asignar al gestor</param>
        /// <returns>True en caso que la operación se ha realizado con éxito, False en caso contrario.</returns>
        [OperationContract]
        bool CambiarPassword(Usuario gestor, string password);

        /// <summary>
        /// Informa si un login de usuario está libre o ya ha sido escogido
        /// </summary>
        /// <param name="login">login a comprobar</param>
        /// <returns>Devuelve true si el login está libre, false en caso contrario</returns>
        [OperationContract]
        bool IsLoginFree(string login);

            /// <summary>
        /// Devuelve un listado con los usuarios del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="nombre">Filtro por nombre del usuario</param>
        /// <param name="tipo">Filtro por tipo de usuario (Gestor/Perito)</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los usuarios del sistema que cumplen el filtro</returns>
        [OperationContract]
        List<Usuario> ListUsuarios(Usuario gestor, string nombre, int tipo, int baja, int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve un listado con los usuarios del sistema de alta
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los usuarios del sistema y que están de alta</returns>
        [OperationContract]
        List<Usuario> ListAllUsuarios(Usuario gestor);

            /// <summary>
        /// Devuelve el número de usuarios que existen en el sistema
        /// </summary>
        /// <returns>Un valor entero con el número total de usuarios del sistema</returns>
        [OperationContract]
        int GetUsuariosCount();

        /// <summary>
        /// Devuelve el número de usuarios que cumplen con los filtros
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="nombre">Filtro por nombre del usuario</param>
        /// <param name="tipo">Filtro por tipo de usuario (Gestor/Perito)</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de usuarios que cumplen el filtro</returns>
        [OperationContract]
        int GetUsuariosCountByFilter(string nombre, int tipo, int baja);

        /// <summary>
        /// Actualiza los datos de un usuario
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Usuario a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el usuario, false en caso contrario</returns>
        [OperationContract]
        bool UpdateUsuario(Usuario gestor, Usuario usuario);

        /// <summary>
        /// Da de baja lógica a un usuario
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Usuario a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el usuario, false en caso contrario</returns>
        [OperationContract]
        bool DeleteUsuario(Usuario gestor, int idUsuario);        
        
        #endregion

        #region Clientes
        /// <summary>
        /// Devuelve un listado con los clientes del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los clientes del sistema</returns>
        [OperationContract]
        List<Cliente> ListClientes(Usuario gestor, string nombre, int baja, int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve un listado con los clientes del sistema de alta
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los clientes del sistema y que están de alta</returns>
        [OperationContract]
        List<Cliente> ListAllClientes(Usuario gestor);

            /// <summary>
        /// Devuelve el número de clientes que existen en el sistema
        /// </summary>
        /// <returns>Un valor entero con el número total de clientes del sistema</returns>
        [OperationContract]
        int GetClientesCount();

        /// <summary>
        /// Devuelve el número de clientes que cumplen con los filtros
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="nombre">Filtro por nombre del cliente</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de clientes que cumplen el filtro</returns>
        [OperationContract]
        int GetClientesCountByFilter(string nombre, int baja);

        /// <summary>
        /// Actualiza los datos de un cliente
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Cliente a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el cliente, false en caso contrario</returns>
        [OperationContract]
        bool UpdateCliente(Usuario gestor, Cliente cliente);

        /// <summary>
        /// Da de baja lógica a un cliente
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Cliente a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el cliente, false en caso contrario</returns>
        [OperationContract]
        bool DeleteCliente(Usuario gestor, int idCliente);

        #endregion

        #region Propiedades

        /// <summary>
        /// Devuelve un listado con los propiedades del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las propiedades</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los propiedades del sistema que cumplen el filtro</returns>
        [OperationContract]
        List<Propiedad> ListPropiedades(Usuario gestor, int idCliente, int baja, int startIndex,
            int count, string sorting);

        /// <summary>
        /// Devuelve un listado con las propiedades de un cliente 
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las propiedades</param>
        /// <returns>Un listado con todas los propiedades activas de un cliente</returns>
        [OperationContract]
        List<Propiedad> ListPropiedadesByCliente(Usuario gestor, int idCliente);

        /// <summary>
        /// Devuelve el número de propiedades que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de propiedades del sistema de un cliente</returns>
        [OperationContract]
        int GetPropiedadesCount(int idCliente);

        /// <summary>
        /// Devuelve el número de propiedades que cumplen con los filtros
        /// </summary>
        /// <param name="idCliente">Cliente del que se desean obtener las propiedades</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de propiedades que cumplen el filtro</returns>
        [OperationContract]
        int GetPropiedadesCountByFilter(int idCliente, int baja);

        /// <summary>
        /// Devuelve un propiedad mediante su identificador único
        /// </summary>
        /// <param name="gestor">Propiedad gestor que solicita la operación</param>
        /// <param name="idPropiedad">Identificador del propiedad</param>
        /// <returns>El propiedad solicitado</returns>
        [OperationContract]
        Propiedad GetPropiedad(Usuario gestor, Int32 idPropiedad);

        /// <summary>
        /// Actualiza los datos de un propiedad
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Propiedad a actualizar</param>
        /// <returns>Devuelve la propiedad que se ha insertado/actualizado</returns>
        [OperationContract]
        Propiedad UpdatePropiedad(Usuario gestor, Propiedad propiedad);

        /// <summary>
        /// Da de baja lógica a un propiedad
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Propiedad a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el propiedad, false en caso contrario</returns>
        [OperationContract]
        bool DeletePropiedad(Usuario gestor, int idPropiedad);

        #endregion

        #region Polizas
        /// <summary>
        /// Devuelve un listado con los polizas del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las polizas</param>
        /// <param name="idPropiedad">Propiedad de la que se desean obtener las polizas</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los polizas del sistema que cumplen el filtro</returns>
        [OperationContract]
        List<vPropiedadPoliza> ListPolizas(Usuario gestor, int idCliente, int idPropiedad, int baja, int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve un listado con las pólizas de un cliente 
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las pólizas</param>
        /// <returns>Un listado con todas los pólizas activas de un cliente</returns>
        [OperationContract]
        List<Poliza> ListPolizasByCliente(Usuario gestor, int idCliente);

        /// <summary>
        /// Devuelve el número de polizas que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de polizas del sistema de un cliente</returns>
        [OperationContract]
        int GetPolizasCount(int idCliente);

        /// <summary>
        /// Devuelve el número de polizas que cumplen con los filtros
        /// </summary>
        /// <param name="nombre">Filtro por nombre del poliza</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de polizas que cumplen el filtro</returns>
        [OperationContract]
        int GetPolizasCountByFilter(int idCliente, int idPropiedad, int baja);

            /// <summary>
        /// Devuelve un poliza mediante su identificador único
        /// </summary>
        /// <param name="gestor">Poliza gestor que solicita la operación</param>
        /// <param name="idPoliza">Identificador del poliza</param>
        /// <returns>El poliza solicitado</returns>
        [OperationContract]
        Poliza GetPoliza(Usuario gestor, Int32 idPoliza);

        /// <summary>
        /// Actualiza los datos de un poliza
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Poliza a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el poliza, false en caso contrario</returns>
        [OperationContract]
        Poliza UpdatePoliza(Usuario gestor, Poliza poliza);

        /// <summary>
        /// Da de baja lógica a un poliza
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Poliza a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el poliza, false en caso contrario</returns>
        [OperationContract]
        bool DeletePoliza(Usuario gestor, int idPoliza);

        #endregion

        #region Ordenes de trabajo
        /// <summary>
        /// Devuelve un listado con los ordenes del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las ordenes</param>
        /// <param name="idPerito">Perito del que se desean obtener las ordenes</param>
        /// <param name="estado">Filtro por estado de la orden</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los ordenes del sistema que cumplen el filtro</returns>
        [OperationContract]
        List<vOrdenTrabajo> ListOrdenes(Usuario gestor, int idCliente, int idPerito, int estado, int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve el número de ordenes que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de ordenes del sistema de un cliente</returns>
        [OperationContract]
        int GetOrdenesCount(int idCliente);

        /// <summary>
        /// Devuelve el número de ordenes que cumplen con los filtros
        /// </summary>
        /// <param name="idCliente">Cliente del que se desean obtener las ordenes</param>
        /// <param name="idPerito">Perito del que se desean obtener las ordenes</param>
        /// <param name="estado">Filtro por estado de la orden</param>
        /// <returns>Un valor entero con el número de ordenes que cumplen el filtro</returns>
        [OperationContract]
        int GetOrdenesCountByFilter(int idCliente, int idPerito, int estado);

        /// <summary>
        /// Devuelve un orden mediante su identificador único
        /// </summary>
        /// <param name="gestor">Orden gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Identificador del orden</param>
        /// <returns>El orden solicitado</returns>
        [OperationContract]
        OrdenTrabajo GetOrden(Usuario gestor, Int32 idOrdenTrabajo);

        /// <summary>
        /// Actualiza los datos de un orden
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="orden">Orden a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el orden, false en caso contrario</returns>
        [OperationContract]
        OrdenTrabajo UpdateOrden(Usuario gestor, OrdenTrabajo orden);

        /// <summary>
        /// Da de baja a un orden
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Identificador del orden a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el orden, false en caso contrario</returns>
        [OperationContract]
        bool DeleteOrden(Usuario gestor, int idOrdenTrabajo);


        /// <summary>
        /// Devuelve el fichero de la foto
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Identificador de la orden para recuperar la foto</param>
        /// <returns>Un array de bytes que contiene el fichero de la foto</returns>
        [OperationContract]
        byte[] GetFoto(Usuario gestor, int idOrdenTrabajo);

        #endregion

        #region Informes
        /// <summary>
        /// Devuelve el informe de ratio de siniestralidad, por año y cliente. El ratio de siniestralidad nos informa del porcentaje
        ///	de órdenes que sobre el total tiene cada cliente.
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="anno">Año para el que solicita el listado</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los clientes que han tenido siniestros (órdenes) en ese año, indicando su ratio de siniestralidad.</returns>
        [OperationContract]
        List<vRatioSiniestralidad> ListRatioSiniestralidad(Usuario gestor, int anno,
            int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve el número de filas que aparecen en un informe de ratio de siniestralidad para un año concreto
        /// </summary>
        /// <param name="anno">Año del que se desean obtener el informe</param>
        /// <returns>Un valor entero con el número de filas que aparecen en un informe de ratio de siniestralidad para un año concreto</returns>
        [OperationContract]
        int GetRatioSiniestralidadCountByFilter(int anno);

        /// <summary>
        /// Devuelve el informe de ratio de productividad, por año y perito. El ratio de productividad nos informa del porcentaje
        ///	de órdenes que sobre el total tiene cada perito.
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="anno">Año para el que solicita el listado</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los peritos que han gestionado siniestros (órdenes) en ese año, indicando su ratio de productividad.</returns>
        [OperationContract]
        List<vRatioProductividad> ListRatioProductividad(Usuario gestor, int anno,
            int startIndex, int count, string sorting);

        /// <summary>
        /// Devuelve el número de filas que aparecen en un informe de ratio de productividad para un año concreto
        /// </summary>
        /// <param name="anno">Año del que se desean obtener el informe</param>
        /// <returns>Un valor entero con el número de filas que aparecen en un informe de ratio de productividad para un año concreto</returns>
        [OperationContract]
        int GetRatioProductividadCountByFilter(int anno);
        
        /// <summary>
        /// Devuelve un listado con los años de los que se tienen registradas órdenes de trabajo valoradas
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los años de los que se tienen registradas órdenes valoradas.</returns>
        [OperationContract]
        List<int?> ListAnnos(Usuario gestor);
        #endregion

    }

}
