﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Web;
using System.Text;
using System.IO;
using SGPM.WebServices.ModeloDatos;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;

namespace SGPM.WebServices
{
    /// <summary>
    /// Implementación del interfaz IWSPhonePer del servicio web para la aplicación PhonePER
    /// </summary>
    public class WSPhonePER : IWSPhonePER
    {

        #region Constructor

        public WSPhonePER()
        {
            // Inicialización del log
            Tracer.Logger.Initialize(ConfigurationManager.AppSettings["Log4NetLoggerNameWSPhonePER"]);

        }

        #endregion

        #region Usuarios
        /// <summary>
        /// Realiza el control de acceso a la aplicación de un usuario Perito
        /// </summary>
        /// <param name="login">Nombre de usuario del perito</param>
        /// <param name="password">Contraseña del perito</param>
        /// <returns>El perito que ha accedido correctamente a la aplicación</returns>
        public PeritoPhone ValidaUsuario(string login, string password)
        {
            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                //Deshabilitamos la carga perezosa y la creación de proxy
                _DataContext.ContextOptions.LazyLoadingEnabled = false;
                _DataContext.ContextOptions.ProxyCreationEnabled = false;

                return _DataContext.PeritoPhone.SingleOrDefault(u => 
                    u.Login == login && 
                    u.Password == password);
            }
        }

        /// <summary>
        /// Realiza el cambio de contraseña de un usuario Gestor
        /// </summary>
        /// <param name="perito">Usuario perito del que se quiere cambiar la constraseña</param>
        /// <param name="password">La contraseña a asignar al gestor</param>
        /// <returns>True en caso que la operación se ha realizado con éxito, False en caso contrario.</returns>
        public bool CambiarPassword(PeritoPhone perito, string password)
        {
            // Se comprueba si el usuario está validado
            if (ValidaUsuario(perito.Login, perito.Password) == null)
            {
                // no está validado
                return false;
            }

            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                //Deshabilitamos la carga perezosa y la creación de proxy
                _DataContext.ContextOptions.LazyLoadingEnabled = false;
                _DataContext.ContextOptions.ProxyCreationEnabled = false;

                Usuario usuario = _DataContext.Usuario.Single(u => u.IdUsuario == perito.IdPerito);
                usuario.Password = password;
                _DataContext.SaveChanges();
                return true;
            }
        
        }

        #endregion

        #region Ordenes de trabajo

        /// <summary>
        /// Devuelve un listado con todas las órdenes de trabajo asignadas a un perito y que están pendientes de recibir su valoración 
        /// </summary>
        /// <param name="perito">Usuario perito que solicita la operación</param>
        /// <returns>Un listado con todas las órdenes de trabajo asignadas al perito y que todavía no se han valorado</returns>
        public List<OrdenTrabajoPhone> ListOrdenes(PeritoPhone perito)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(perito.Login, perito.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    //Extraemos las órdenes que se van a descargar. En el listado de órdenes se incluyen
                    //las que se encuentren en estado Asignada y Descargada. Estas últimas se adjuntan para
                    //que se puedan eliminar las que en su momento se asignaron al perito pero se le han 
                    //desasignado
                    var ordenesPhone = _DataContext.OrdenTrabajoPhone.Where(o => o.IdPerito == perito.IdPerito
                        && o.IdEstado != ((int)EstadosOrden.Cerrada)).ToList();
                    var ordenes = _DataContext.OrdenTrabajo.Where(o => o.IdPerito == perito.IdPerito
                        && o.IdEstado != ((int)EstadosOrden.Cerrada));

                    //Actualizamos el estado
                    foreach (OrdenTrabajo orden in ordenes)
                    {
                        //Transitamos el estado a Descargada
                        orden.IdEstado = (int)EstadosOrden.Descargada;
                        _DataContext.ApplyCurrentValues<OrdenTrabajo>("OrdenTrabajo", orden);
                    }
                    _DataContext.SaveChanges();

                    //Devuelve todas las órdenes vinculadas a un perito, que todavía no se hayan cerrado
                    return ordenesPhone;

                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza una orden de trabajo con la valoración del perito. La orden pasa a estado Cerrada
        /// </summary>
        /// <param name="perito">Usuario perito que solicita la operación</param>
        /// <param name="orden">Orden de trabajo que se ha valorado</param>
        /// <param name="pictureStream">Array de bytes que contiene la foto anexa</param>
        /// <returns>True en caso que la operación se haya efecturado correctamente, False en caso contratio</returns>
        public bool UpdateOrden(PeritoPhone perito, OrdenTrabajoPhone orden, byte[] pictureStream)
        {
            //Nos informa del resultado de la operaciones
            bool bOk = false;

            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(perito.Login, perito.Password) == null)
                {
                    return false;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    //Recupera la orden de trabajo
                    OrdenTrabajo existingObject = _DataContext.OrdenTrabajo.Where(x => x.IdOrdenTrabajo == orden.IdOrdenTrabajo).FirstOrDefault();

                    //Se comprueba si el item no existe en la base de datos, o si ya se ha cerrado
                    if (existingObject == null || existingObject.IdEstado==(int)EstadosOrden.Cerrada)
                    {
                        //Error, situación inconsistente. Se ha pasado una PK que no existe
                        throw new Exception(string.Format("El idOrdenTrabajo {0} no existe o ya está en estado Cerrada.", orden.IdOrdenTrabajo));
                    }

                    // Primero: se adjunta la foto
                    if (pictureStream != null)
                    {
                        // Inicializamos el nombre del fichero y su ruta
                        string fileName = string.Format("foto_{0}.jpeg", orden.IdOrdenTrabajo);

                        bOk = UploadFoto(fileName, pictureStream);

                        if (!bOk)
                        {
                            // Se ha producido un error al subir el archivo
                            throw new Exception(string.Format("No se ha podido adjunta la imagen de la orden:{0}",
                                orden.IdOrdenTrabajo));
                        }

                        // Insertamos la referencia a la foto en la base de datos
                        Foto foto = new Foto();
                        foto.IdOrdenTrabajo = orden.IdOrdenTrabajo;
                        foto.Ruta = fileName;
                        _DataContext.Foto.AddObject(foto);
                        
                    }

                    //Segundo: Actualiza la orden de trabajo
                    existingObject.IdEstado = (int)EstadosOrden.Cerrada;
                    existingObject.Aceptada = orden.Aceptada;
                    existingObject.Descripcion = orden.Descripcion;
                    existingObject.FechaValoracion = orden.FechaValoracion;

                    _DataContext.ApplyCurrentValues<OrdenTrabajo>("OrdenTrabajo", existingObject);
                    _DataContext.SaveChanges();

                    return true;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Almacena en un directorio del servicio web la imagen adjunta a una orden de trabajo
        /// </summary>
        /// <param name="fileName">Nombre del fichero, donde se almacenara la imagen</param>
        /// <param name="pictureStream">Array de bytes que contiene la foto anexa</param>
        /// <returns>True en caso que la operación se haya efecturado correctamente, False en caso contratio</returns>
        private bool UploadFoto(string fileName, byte[] pictureStream)
        {
            FileStream fileStream = null;
            BinaryWriter writer = null;

            try
            {
                if (pictureStream != null)
                {
                    string filePath = string.Concat(ConfigurationManager.AppSettings["PictureUploadPath"],
                               fileName);

                    fileStream = File.Open(filePath, FileMode.Create);
                    writer = new BinaryWriter(fileStream);
                    writer.Write(pictureStream);
                }

                return true;
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
            finally
            {
                if (fileStream != null)
                    fileStream.Close();
                if (writer != null)
                    writer.Close();
            }
        }
        #endregion

        #region Notificaciones Toast

        /// <summary>
        /// Recibe la URI que el Servicio de Notificaciones Push de Microsoft devuelve al cliente push cuando se
        /// crea el canal de notificaciones. De esta forma se vincula la URI de subscripción al dispositivo del perito.
        /// </summary>
        /// <param name="perito">Usuario perito al que se le vincula la URI</param>
        /// <param name="subscriptionURI">URI de subscripción al servicio de notificaciones push</param>
        /// <returns>Devuelve true si ha sido posible recepcionar la URI, false en caso contrario</returns>
        public bool SetSubscriptionURI(PeritoPhone perito, string subscriptionURI)
        {
            // Se comprueba si el usuario está validado
            if (ValidaUsuario(perito.Login, perito.Password) == null)
            {
                // no está validado
                return false;
            }

            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                //Deshabilitamos la carga perezosa y la creación de proxy
                _DataContext.ContextOptions.LazyLoadingEnabled = false;
                _DataContext.ContextOptions.ProxyCreationEnabled = false;

                Usuario usuario = _DataContext.Usuario.Single(u => u.IdUsuario == perito.IdPerito);
                usuario.SubscriptionURI  = subscriptionURI;
                _DataContext.SaveChanges();
                return true;
            }
        }
        #endregion

    }
}
