﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using SGPM.WebServices;
using SGPM.WebServices.ModeloDatos;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;

using System.Net;
using System.IO;

namespace SGPM.WebServices
{
    /// <summary>
    /// Implementación del interfaz IWSWebPer del servicio web para la aplicación WebPER
    /// </summary>
    public class WSWebPER : IWSWebPER
    {

        #region Constructor

        public WSWebPER()
        {
            // Inicialización del log
            Tracer.Logger.Initialize(ConfigurationManager.AppSettings["Log4NetLoggerNameWSWebPER"]);
        }

        #endregion

        #region Provincias
        /// <summary>
        /// Devuelve un listado con las provincias del sistema
        /// </summary>
        /// <returns>Un listado con todas las provincias</returns>
        public List<Provincia> ListProvincias(Usuario gestor)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    return _DataContext.Provincia.OrderBy(p => p.Nombre).ToList() ;

                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        #endregion
        
        #region Tipos de siniestro
        /// <summary>
        /// Devuelve un listado con los tipos de siniestro del sistema
        /// </summary>
        /// <returns>Un listado con todos los tipos de siniestro</returns>
        public List<TipoSiniestro> ListTiposSiniestro(Usuario gestor)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    return _DataContext.TipoSiniestro.OrderBy(p => p.Nombre).ToList();

                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        #endregion

        #region Estados de la orden de trabajo
        /// <summary>
        /// Devuelve un listado con los estados de la orden de trabajo
        /// </summary>
        /// <returns>Un listado con todos las estados de la orden de trabajo</returns>
        public List<EstadoOrden> ListEstadosOrden(Usuario gestor)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    return _DataContext.EstadoOrden.OrderBy(p => p.IdEstado).ToList();

                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        #endregion

        #region Usuarios
        /// <summary>
        /// Realiza el control de acceso a la aplicación de un usuario Gestor
        /// </summary>
        /// <param name="login">Nombre de usuario del gestor</param>
        /// <param name="password">Contraseña del gestor</param>
        /// <returns>El gestor que ha accedido correctamente a la aplicación</returns>
        public Usuario ValidaUsuario(string login, string password)
        {
            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                //Deshabilitamos la carga perezosa y la creación de proxy
                _DataContext.ContextOptions.LazyLoadingEnabled = false;
                _DataContext.ContextOptions.ProxyCreationEnabled = false;

                Usuario gestor = _DataContext.Usuario.SingleOrDefault(g => 
                    g.Login == login &&
                    g.Password == password && 
                    g.Tipo == (int)TipoUsuario.Gestor &&
                    g.FechaBaja == null);

                return gestor;
            }
        }

        /// <summary>
        /// Realiza el cambio de contraseña de un usuario Gestor
        /// </summary>
        /// <param name="gestor">Usuario gestor del que se quiere cambiar la constraseña</param>
        /// <param name="password">La contraseña a asignar al gestor</param>
        /// <returns>True en caso que la operación se ha realizado con éxito, False en caso contrario.</returns>
        public bool CambiarPassword(Usuario gestor, string password)
        {
            // Se comprueba si el usuario está validado
            if (ValidaUsuario(gestor.Login, gestor.Password) == null)
            {
                // no está validado
                return false;
            }

            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                //Deshabilitamos la carga perezosa y la creación de proxy
                _DataContext.ContextOptions.LazyLoadingEnabled = false;
                _DataContext.ContextOptions.ProxyCreationEnabled = false;

                Usuario usuario = _DataContext.Usuario.Single(u => u.IdUsuario == gestor.IdUsuario);
                usuario.Password = password;
                _DataContext.SaveChanges();
                return true;
            }
        }

        /// <summary>
        /// Informa si un login de usuario está libre o ya ha sido escogido
        /// </summary>
        /// <param name="login">login a comprobar</param>
        /// <returns>Devuelve true si el login está libre, false en caso contrario</returns>
        public bool IsLoginFree(string login)
        {
            bool bResult = false;

            using (SGPMEntities _DataContext = new SGPMEntities())
            {
                bResult = (_DataContext.Usuario.Where(x => x.Login == login).Count() == 0);
            }

            return bResult;
        }


        /// <summary>
        /// Devuelve un listado con los usuarios del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="nombre">Filtro por nombre del usuario</param>
        /// <param name="tipo">Filtro por tipo de usuario (Gestor/Perito)</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los usuarios del sistema que cumplen el filtro</returns>
        public List<Usuario> ListUsuarios(Usuario gestor, string nombre, int tipo, int baja, int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Usuario> query = _DataContext.Usuario;

                    // Filtro por nombre
                    if (!String.IsNullOrEmpty(nombre))
                    {
                        query = query.Where(s => s.Nombre.IndexOf(nombre, StringComparison.OrdinalIgnoreCase) >= 0);
                    }

                    // Filtro por tipo de usuario
                    if (tipo != (int)TipoUsuario.Todos)
                    {
                        query = query.Where(s => s.Tipo.Equals(tipo));
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("Nombre ASC"))
                    {
                        query = query.OrderBy(p => p.Nombre);
                    }
                    else if (sorting.Equals("Nombre DESC"))
                    {
                        query = query.OrderByDescending(p => p.Nombre);
                    }
                    else if (sorting.Equals("Apellidos ASC"))
                    {
                        query = query.OrderBy(p => p.Apellidos);
                    }
                    else if (sorting.Equals("Apellidos DESC"))
                    {
                        query = query.OrderByDescending(p => p.Apellidos);
                    }
                    else if (sorting.Equals("Tipo ASC"))
                    {
                        query = query.OrderBy(p => p.Tipo);
                    }
                    else if (sorting.Equals("Tipo DESC"))
                    {
                        query = query.OrderByDescending(p => p.Tipo);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.Nombre); //Por defecto!
                    }

                    // Retorna la lista de usuarios
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve un listado con los usuarios del sistema de alta
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los usuarios del sistema y que están de alta</returns>
        public List<Usuario> ListAllUsuarios(Usuario gestor)
        {
            return ListUsuarios(gestor, null, 0, (int)FiltroEstadoBaja.Activo, 0, 0, null);
        }

        /// <summary>
        /// Devuelve el número de usuarios que existen en el sistema
        /// </summary>
        /// <returns>Un valor entero con el número total de usuarios del sistema</returns>
        public int GetUsuariosCount()
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    return _DataContext.Usuario.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1; 
            }
        }


        /// <summary>
        /// Devuelve el número de usuarios que cumplen con los filtros
        /// </summary>
        /// <param name="nombre">Filtro por nombre del usuario</param>
        /// <param name="tipo">Filtro por tipo de usuario (Gestor/Perito)</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de usuarios que cumplen el filtro</returns>
        public int GetUsuariosCountByFilter(string nombre, int tipo, int baja)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Usuario> query = _DataContext.Usuario;

                    // Filtro por nombre
                    if (!String.IsNullOrEmpty(nombre))
                    {
                        query = query.Where(s => s.Nombre.IndexOf(nombre, StringComparison.OrdinalIgnoreCase) >= 0);
                    }

                    // Filtro por tipo de usuario
                    if (tipo != (int)TipoUsuario.Todos)
                    {
                        query = query.Where(s => !s.Tipo.Equals(tipo));
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }
    

        /// <summary>
        /// Devuelve un usuario mediante su identificador único
        /// </summary>
        /// <param name="gestor">usuario gestor que solicita la operación</param>
        /// <param name="idusuario">Identificador del usuario</param>
        /// <returns>El usuario solicitado</returns>
        public Usuario GetUsuario(Usuario gestor, Int32 idUsuario)
        {
            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    // no está validado
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    // Se retorna el usuario por el que se interroga
                    return _DataContext.Usuario.SingleOrDefault(usuario => usuario.IdUsuario == idUsuario);
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza los datos de un usuario
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">usuario a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el usuario, false en caso contrario</returns>
        public bool UpdateUsuario(Usuario gestor, Usuario usuario)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null) {

                    using (SGPMEntities _DataContext = new SGPMEntities()) {

                        Usuario existingObject = _DataContext.Usuario.Where(x => x.IdUsuario == usuario.IdUsuario).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null && usuario.IdUsuario > 0)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(String.Format ("El idusuario {0} no existe.", usuario.IdUsuario));
                        }
                        //Si el item no tiene PK, entonces es un alta
                        else if (usuario.IdUsuario <= 0)
                        {
                            // Comprobamos que no exista el login
                            if (_DataContext.Usuario.Where(x => x.Login == usuario.Login).Count() > 0)
                            {
                                // El login ya existe, no se puede permitir su inserción
                                throw new Exception(String.Format("El login {0} ya existe.", usuario.Login));
                            }

                            //Actualizamos los datos de la auditoría de cambios
                            usuario.FechaAlta = DateTime.Today;
                            usuario.UAlta = gestor.IdUsuario;
                            _DataContext.Usuario.AddObject(usuario);
                        }
                        //El item existe, y posee un ID, entonces es una actualización
                        else
                        {
                            //Inicializamos los datos que no se han pasado
                            usuario.FechaAlta = existingObject.FechaAlta;
                            usuario.UAlta = existingObject.UAlta;
                            _DataContext.ApplyCurrentValues<Usuario>("Usuario", usuario);
                        }
                
                        _DataContext.SaveChanges();
                    }

                    return true;
                } else {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Da de baja lógica a un usuario
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idUsuario">usuario a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el usuario, false en caso contrario</returns>
        public bool DeleteUsuario(Usuario gestor, int idUsuario)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Usuario existingObject = _DataContext.Usuario.Where(x => x.IdUsuario == idUsuario).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(String.Format("El idusuario {0} no existe.",idUsuario));
                        }

                        //Validación del borrado.
                        //Se comprueba que el usuario (si es perito) no tenga ordenes de trabajo en estado Descargada
                        if (existingObject.OrdenPerito.Where(o => o.EstadoOrden.IdEstado == (int)EstadosOrden.Descargada).Count() > 0)
                        {
                            //Error, no se puede eliminar el usuario
                            throw new Exception(String.Format("El idusuario {0} es perito y tiene ordenes descargadas.", idUsuario));
                        }

                        //Actualizamos los datos de la baja lógica y auditoría de cambios
                        existingObject.FechaBaja = DateTime.Today;
                        existingObject.UBaja = gestor.IdUsuario;

                        _DataContext.ApplyCurrentValues<Usuario>("Usuario", existingObject);

                        _DataContext.SaveChanges();
                    }

                    return true;
                }
                else
                {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        #endregion

        #region Clientes
        /// <summary>
        /// Devuelve un listado con los clientes del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="nombre">Filtro por nombre del cliente</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los clientes del sistema que cumplen el filtro</returns>
        public List<Cliente> ListClientes(Usuario gestor, string nombre, int baja, int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null) {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities()) {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Cliente> query = _DataContext.Cliente;

                    // Filtros
                    if (!String.IsNullOrEmpty(nombre))
                    {
                        query = query.Where(s => s.Nombre.IndexOf(nombre, StringComparison.OrdinalIgnoreCase) >= 0);
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("Nombre ASC"))
                    {
                        query = query.OrderBy(p => p.Nombre);
                    }
                    else if (sorting.Equals("Nombre DESC"))
                    {
                        query = query.OrderByDescending(p => p.Nombre);
                    }
                    else if (sorting.Equals("Apellidos ASC"))
                    {
                        query = query.OrderBy(p => p.Apellidos);
                    }
                    else if (sorting.Equals("Apellidos DESC"))
                    {
                        query = query.OrderByDescending(p => p.Apellidos);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.Nombre); //Por defecto!
                    }

                    // Retorna la lista de clientes
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }


        /// <summary>
        /// Devuelve un listado con los clientes del sistema de alta
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los clientes del sistema y que están de alta</returns>
        public List<Cliente> ListAllClientes(Usuario gestor)
        {
            return ListClientes(gestor, null, (int)FiltroEstadoBaja.Activo, 0, 0, "Nombre");
        }

        /// <summary>
        /// Devuelve el número de clientes que existen en el sistema
        /// </summary>
        /// <returns>Un valor entero con el número total de clientes del sistema</returns>
        public int GetClientesCount()
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    return _DataContext.Cliente.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1; 
            }
        }


        /// <summary>
        /// Devuelve el número de clientes que cumplen con los filtros
        /// </summary>
        /// <param name="nombre">Filtro por nombre del cliente</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de clientes que cumplen el filtro</returns>
        public int GetClientesCountByFilter(string nombre, int baja)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Cliente> query = _DataContext.Cliente;

                    // Filtros
                    if (!String.IsNullOrEmpty(nombre))
                    {
                        query = query.Where(s => s.Nombre.IndexOf(nombre, StringComparison.OrdinalIgnoreCase) >= 0);
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }
    

        /// <summary>
        /// Devuelve un cliente mediante su identificador único
        /// </summary>
        /// <param name="gestor">Cliente gestor que solicita la operación</param>
        /// <param name="idCliente">Identificador del cliente</param>
        /// <returns>El cliente solicitado</returns>
        public Cliente GetCliente(Usuario gestor, Int32 idCliente)
        {
            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    // no está validado
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    // Se retorna el cliente por el que se interroga
                    return _DataContext.Cliente.SingleOrDefault(cliente => cliente.IdCliente == idCliente);
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza los datos de un cliente
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Cliente a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el cliente, false en caso contrario</returns>
        public bool UpdateCliente(Usuario gestor, Cliente cliente)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null) {

                    using (SGPMEntities _DataContext = new SGPMEntities()) {

                        Cliente existingObject = _DataContext.Cliente.Where(x => x.IdCliente == cliente.IdCliente).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null && cliente.IdCliente > 0)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception("El idCliente " + cliente.IdCliente + " no existe.");
                        }
                        //Si el item no tiene PK, entonces es un alta
                        else if(cliente.IdCliente <= 0)
                        {
                            //Actualizamos los datos de la auditoría de cambios
                            cliente.FechaAlta = DateTime.Today;
                            cliente.UAlta = gestor.IdUsuario;
                            _DataContext.Cliente.AddObject(cliente);
                        }
                        //El item existe, y posee un ID, entonces es una actualización
                        else
                        {
                            //Inicializamos los datos que no se han pasado
                            cliente.FechaAlta = existingObject.FechaAlta;
                            cliente.UAlta = existingObject.UAlta;
                            _DataContext.ApplyCurrentValues<Cliente>("Cliente", cliente);
                        }
                
                        _DataContext.SaveChanges();
                    }

                    return true;
                } else {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Da de baja lógica a un cliente
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Cliente a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el cliente, false en caso contrario</returns>
        public bool DeleteCliente(Usuario gestor, int idCliente)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Cliente existingObject = _DataContext.Cliente.Where(x => x.IdCliente == idCliente).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(String.Format("El idCliente {0} no existe.", idCliente));
                        }

                        //Actualizamos los datos de la baja lógica y auditoría de cambios
                        existingObject.FechaBaja = DateTime.Today;
                        existingObject.UBaja = gestor.IdUsuario;

                        _DataContext.ApplyCurrentValues<Cliente>("Cliente", existingObject);

                        _DataContext.SaveChanges();
                    }

                    return true;
                }
                else
                {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        #endregion

        #region Propiedades
        /// <summary>
        /// Devuelve un listado con los propiedades del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las propiedades</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los propiedades del sistema que cumplen el filtro</returns>
        public List<Propiedad> ListPropiedades(Usuario gestor, int idCliente, int baja, int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Propiedad> query = _DataContext.Propiedad;

                    // Filtros
                    query = query.Where(s => s.IdCliente == idCliente);

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("Localidad ASC"))
                    {
                        query = query.OrderBy(p => p.Localidad);
                    }
                    else if (sorting.Equals("Localidad DESC"))
                    {
                        query = query.OrderByDescending(p => p.Localidad);
                    }
                    else if (sorting.Equals("Provincia ASC"))
                    {
                        query = query.OrderBy(p => p.Provincia);
                    }
                    else if (sorting.Equals("Provincia DESC"))
                    {
                        query = query.OrderByDescending(p => p.Provincia);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.Provincia); //Por defecto!
                    }

                    // Retorna la lista de propiedades
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve un listado con las propiedades de un cliente 
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las propiedades</param>
        /// <returns>Un listado con todas los propiedades activas de un cliente</returns>
        public List<Propiedad> ListPropiedadesByCliente(Usuario gestor, int idCliente)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Propiedad> query = _DataContext.Propiedad.Include("Provincia");

                    // Filtros
                    query = query.Where(s => s.IdCliente == idCliente && s.FechaBaja.Equals(null));

                    // Ordenación
                    query = query.OrderBy(p => p.Provincia.Nombre);

                    // Retorna la lista de propiedades
                    return query.ToList(); 
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve el número de propiedades que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de propiedades del sistema de un cliente</returns>
        public int GetPropiedadesCount(int idCliente)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    return _DataContext.Propiedad.Where(s => s.IdCliente == idCliente).Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve el número de propiedades que cumplen con los filtros
        /// </summary>
        /// <param name="nombre">Filtro por nombre del propiedad</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de propiedades que cumplen el filtro</returns>
        public int GetPropiedadesCountByFilter(int idCliente, int baja)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Propiedad> query = _DataContext.Propiedad;

                    // Filtros
                    query = query.Where(s => s.IdCliente == idCliente);

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve un propiedad mediante su identificador único
        /// </summary>
        /// <param name="gestor">Propiedad gestor que solicita la operación</param>
        /// <param name="idPropiedad">Identificador del propiedad</param>
        /// <returns>El propiedad solicitado</returns>
        public Propiedad GetPropiedad(Usuario gestor, Int32 idPropiedad)
        {
            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    // no está validado
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    // Se retorna el propiedad por el que se interroga
                    return _DataContext.Propiedad.SingleOrDefault(propiedad => propiedad.IdPropiedad == idPropiedad);
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza los datos de un propiedad
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Propiedad a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el propiedad, false en caso contrario</returns>
        public Propiedad UpdatePropiedad(Usuario gestor, Propiedad propiedad)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Propiedad existingObject = _DataContext.Propiedad.Where(x => x.IdPropiedad == propiedad.IdPropiedad).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null && propiedad.IdPropiedad > 0)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(String.Format("El idPropiedad {0} no existe.", propiedad.IdPropiedad));
                        }
                        //Si el item no tiene PK, entonces es un alta
                        else if (propiedad.IdPropiedad <= 0)
                        {
                            //Actualizamos los datos de la auditoría de cambios
                            propiedad.FechaAlta = DateTime.Today;
                            propiedad.UAlta = gestor.IdUsuario;
                            _DataContext.Propiedad.AddObject(propiedad);
                        }
                        //El item existe, y posee un ID, entonces es una actualización
                        else
                        {
                            //Inicializamos los datos que no se han pasado
                            propiedad.IdCliente = existingObject.IdCliente;
                            propiedad.FechaAlta = existingObject.FechaAlta;
                            propiedad.UAlta = existingObject.UAlta;
                            _DataContext.ApplyCurrentValues<Propiedad>("Propiedad", propiedad);
                        }

                        _DataContext.SaveChanges();
                    }

                    return propiedad;
                }
                else
                {
                    // Usuario no logeado
                    return null;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Da de baja lógica a un propiedad
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Propiedad a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el propiedad, false en caso contrario</returns>
        public bool DeletePropiedad(Usuario gestor, int idPropiedad)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Propiedad existingObject = _DataContext.Propiedad.Where(x => x.IdPropiedad == idPropiedad).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(String.Format("El idPropiedad {0} no existe.", idPropiedad));
                        }

                        //Actualizamos los datos de la baja lógica y auditoría de cambios
                        existingObject.FechaBaja = DateTime.Today;
                        existingObject.UBaja = gestor.IdUsuario;

                        _DataContext.ApplyCurrentValues<Propiedad>("Propiedad", existingObject);

                        _DataContext.SaveChanges();
                    }

                    return true;
                }
                else
                {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        #endregion

        #region Polizas
        /// <summary>
        /// Devuelve un listado con los polizas del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las polizas</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los polizas del sistema que cumplen el filtro</returns>
        public List<vPropiedadPoliza> ListPolizas(Usuario gestor, int idCliente, int idPropiedad, int baja, 
            int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vPropiedadPoliza> query = _DataContext.vPropiedadPoliza;

                    // Filtros
                    query = query.Where(s => s.IdCliente == idCliente);
                    if (idPropiedad > 0)
                    {
                        query = query.Where(s => s.IdPropiedad == idPropiedad);
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("NPoliza ASC"))
                    {
                        query = query.OrderBy(p => p.NPoliza);
                    }
                    else if (sorting.Equals("NPoliza DESC"))
                    {
                        query = query.OrderByDescending(p => p.NPoliza);
                    }
                    else if (sorting.Equals("FechaRenovacion ASC"))
                    {
                        query = query.OrderBy(p => p.FechaRenovacion);
                    }
                    else if (sorting.Equals("FechaRenovacion DESC"))
                    {
                        query = query.OrderByDescending(p => p.FechaRenovacion);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.NPoliza); //Por defecto!
                    }

                    // Retorna la lista de polizas
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve un listado con las pólizas de un cliente 
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las pólizas</param>
        /// <returns>Un listado con todas las pólizas activas de un cliente</returns>
        public List<Poliza> ListPolizasByCliente(Usuario gestor, int idCliente)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Poliza> query = _DataContext.Poliza.Include("Propiedad");

                    // Filtros
                    query = query.Where(s => s.Propiedad.IdCliente == idCliente && s.FechaBaja.Equals(null));

                    // Ordenación
                    query = query.OrderBy(s => s.NPoliza);

                    // Retorna la lista de propiedades
                    return query.ToList();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve el número de polizas que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de polizas del sistema de un cliente</returns>
        public int GetPolizasCount(int idCliente)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    return _DataContext.Poliza.Where(s => s.Propiedad.IdCliente == idCliente).Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve el número de polizas que cumplen con los filtros
        /// </summary>
        /// <param name="nombre">Filtro por nombre del poliza</param>
        /// <param name="baja">Filtro por estado de alta o baja</param>
        /// <returns>Un valor entero con el número de polizas que cumplen el filtro</returns>
        public int GetPolizasCountByFilter(int idCliente, int idPropiedad, int baja)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<Poliza> query = _DataContext.Poliza.Include("Propiedad");

                    // Filtros
                    query = query.Where(s => s.Propiedad.IdCliente == idCliente);
                    if (idPropiedad > 0)
                    {
                        query = query.Where(s => s.IdPropiedad == idPropiedad);
                    }

                    //Filtro por baja
                    switch (baja)
                    {
                        case (int)FiltroEstadoBaja.Baja:
                            // los de baja
                            query = query.Where(s => !s.FechaBaja.Equals(null));
                            break;
                        case (int)FiltroEstadoBaja.Activo:
                            // los de alta
                            query = query.Where(s => s.FechaBaja.Equals(null));
                            break;
                    }

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve un poliza mediante su identificador único
        /// </summary>
        /// <param name="gestor">Poliza gestor que solicita la operación</param>
        /// <param name="idPoliza">Identificador del poliza</param>
        /// <returns>El poliza solicitado</returns>
        public Poliza GetPoliza(Usuario gestor, Int32 idPoliza)
        {
            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    // no está validado
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    // Se retorna el poliza por el que se interroga
                    return _DataContext.Poliza.SingleOrDefault(poliza => poliza.IdPoliza == idPoliza);
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza los datos de un poliza
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Poliza a actualizar</param>
        /// <returns>Devuelve true si ha sido posible actualizar el poliza, false en caso contrario</returns>
        public Poliza UpdatePoliza(Usuario gestor, Poliza poliza)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Poliza existingObject = _DataContext.Poliza.Where(x => x.IdPoliza == poliza.IdPoliza).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null && poliza.IdPoliza > 0)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception("El idPoliza " + poliza.IdPoliza + " no existe.");
                        }
                        //Si el item no tiene PK, entonces es un alta
                        else if (poliza.IdPoliza <= 0)
                        {
                            //Actualizamos los datos de la auditoría de cambios
                            poliza.FechaAlta = DateTime.Today;
                            poliza.UAlta = gestor.IdUsuario;
                            _DataContext.Poliza.AddObject(poliza);
                        }
                        //El item existe, y posee un ID, entonces es una actualización
                        else
                        {
                            //Inicializamos los datos que no se han pasado
                            poliza.FechaAlta = existingObject.FechaAlta;
                            poliza.UAlta = existingObject.UAlta;
                            _DataContext.ApplyCurrentValues<Poliza>("Poliza", poliza);
                        }

                        _DataContext.SaveChanges();
                    }

                    return poliza;
                }
                else
                {
                    // Usuario no logeado
                    return null;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Da de baja lógica a un poliza
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="client">Poliza a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible actualizar el poliza, false en caso contrario</returns>
        public bool DeletePoliza(Usuario gestor, int idPoliza)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Poliza existingObject = _DataContext.Poliza.Where(x => x.IdPoliza == idPoliza).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception("El idPoliza " + idPoliza + " no existe.");
                        }

                        //Actualizamos los datos de la baja lógica y auditoría de cambios
                        existingObject.FechaBaja = DateTime.Today;
                        existingObject.UBaja = gestor.IdUsuario;

                        _DataContext.ApplyCurrentValues<Poliza>("Poliza", existingObject);

                        _DataContext.SaveChanges();
                    }

                    return true;
                }
                else
                {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        #endregion

        #region Ordenes
        /// <summary>
        /// Devuelve un listado con los ordenes de trabajo del sistema
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idCliente">Cliente del que se desean obtener las ordenes</param>
        /// <param name="idPerito">Perito del que se desean obtener las ordenes</param>
        /// <param name="idEstado">Filtro por estado de la orden</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los ordenes de trabajo del sistema que cumplen el filtro</returns>
        public List<vOrdenTrabajo> ListOrdenes(Usuario gestor, int idCliente, int idPerito, int idEstado,
            int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vOrdenTrabajo> query = _DataContext.vOrdenTrabajo;
                    // Filtros
                    if (idCliente > 0)
                    {
                        query = query.Where(s => s.IdCliente == idCliente);
                    }
                    if (idPerito > 0)
                    {
                        query = query.Where(s => s.IdPerito == idPerito);
                    }

                    //Filtro por estado
                    if (idEstado > 0)
                    {
                        query = query.Where(s => s.IdEstado == idEstado);
                    }

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("IdOrdenTrabajo ASC"))
                    {
                        query = query.OrderBy(p => p.IdOrdenTrabajo);
                    }
                    else if (sorting.Equals("IdOrdenTrabajo DESC"))
                    {
                        query = query.OrderByDescending(p => p.IdOrdenTrabajo);
                    }
                    else if (sorting.Equals("NombreCliente ASC"))
                    {
                        query = query.OrderBy(p => p.NombreCliente);
                    }
                    else if (sorting.Equals("NombreCliente DESC"))
                    {
                        query = query.OrderByDescending(p => p.NombreCliente);
                    }
                    else if (sorting.Equals("IdPerito ASC"))
                    {
                        query = query.OrderBy(p => p.NombrePerito);
                    }
                    else if (sorting.Equals("IdPerito DESC"))
                    {
                        query = query.OrderByDescending(p => p.NombrePerito);
                    }
                    else if (sorting.Equals("NombreEstado ASC"))
                    {
                        query = query.OrderBy(p => p.NombreEstado);
                    }
                    else if (sorting.Equals("NombreEstado DESC"))
                    {
                        query = query.OrderByDescending(p => p.NombreEstado);
                    }
                    else if (sorting.Equals("Aceptada ASC"))
                    {
                        query = query.OrderBy(p => p.Aceptada);
                    }
                    else if (sorting.Equals("Aceptada DESC"))
                    {
                        query = query.OrderByDescending(p => p.Aceptada);
                    }
                    else if (sorting.Equals("FechaSiniestro ASC"))
                    {
                        query = query.OrderBy(p => p.FechaSiniestro);
                    }
                    else if (sorting.Equals("FechaSiniestro DESC"))
                    {
                        query = query.OrderByDescending(p => p.FechaSiniestro);
                    }
                    else if (sorting.Equals("FechaValoracion ASC"))
                    {
                        query = query.OrderBy(p => p.FechaValoracion);
                    }
                    else if (sorting.Equals("FechaValoracion DESC"))
                    {
                        query = query.OrderByDescending(p => p.FechaValoracion);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.IdOrdenTrabajo); //Por defecto!
                    }

                    // Retorna la lista de ordenes de trabajo
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }


        /// <summary>
        /// Devuelve el número de ordenes de trabajo que existen en el sistema de un cliente determinada
        /// </summary>
        /// <returns>Un valor entero con el número total de ordenes de trabajo del sistema de un cliente</returns>
        public int GetOrdenesCount(int idCliente)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    return _DataContext.vOrdenTrabajo.Where(s => s.IdCliente == idCliente).Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve el número de ordenes de trabajo que cumplen con los filtros
        /// </summary>
        /// <param name="idCliente">Cliente del que se desean obtener las ordenes</param>
        /// <param name="idPerito">Perito del que se desean obtener las ordenes</param>
        /// <param name="idEstado">Filtro por estado de la orden</param>
        /// <returns>Un valor entero con el número de ordenes de trabajo que cumplen el filtro</returns>
        public int GetOrdenesCountByFilter(int idCliente, int idPerito, int idEstado)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vOrdenTrabajo> query = _DataContext.vOrdenTrabajo;

                    // Filtros
                    if (idCliente > 0)
                    {
                        query = query.Where(s => s.IdCliente == idCliente);
                    }
                    if (idPerito > 0)
                    {
                        query = query.Where(s => s.IdPerito == idPerito);
                    }

                    //Filtro por idEstado
                    if (idEstado > 0)
                    {
                        query = query.Where(s => s.IdEstado == idEstado);
                    }

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }


        /// <summary>
        /// Devuelve un orden de trabajo mediante su identificador único
        /// </summary>
        /// <param name="gestor">Poliza gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Identificador de la orden de trabajo</param>
        /// <returns>La orden de trabajo solicitada</returns>
        public OrdenTrabajo GetOrden(Usuario gestor, Int32 idOrdenTrabajo)
        {
            try
            {
                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    // no está validado
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    // Se retorna la orden por la que se interroga
                    return _DataContext.OrdenTrabajo.SingleOrDefault(orden => orden.IdOrdenTrabajo == idOrdenTrabajo);
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Actualiza los datos de una orden de trabajo. También notifica por push a los peritos implicados
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="orden">Orden a actualizar</param>
        /// <returns>Devuelve la orden de trabajo actualizada</returns>
        public OrdenTrabajo UpdateOrden(Usuario gestor, OrdenTrabajo orden)
        {
            int idPeritoAnt = 0;

            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        OrdenTrabajo existingObject = _DataContext.OrdenTrabajo.Where(x => x.IdOrdenTrabajo == orden.IdOrdenTrabajo).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null && orden.IdOrdenTrabajo > 0)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(string.Format("El idOrdenTrabajo {0} no existe.", orden.IdOrdenTrabajo));
                        }
                        //Si el item no tiene PK, entonces es un alta
                        else if (orden.IdOrdenTrabajo <= 0)
                        {
                            // Inicializamos los valores de estado
                            orden.IdGestor = gestor.IdUsuario;
                            if (orden.IdPerito != 0)
                            {
                                //Tiene perito asignado
                                orden.IdEstado = (int)EstadosOrden.Asignada;
                            }
                            else
                            {
                                //No tiene perito asignado
                                orden.IdEstado = (int)EstadosOrden.Registrada;
                                orden.IdPerito = null;
                            }

                            _DataContext.OrdenTrabajo.AddObject(orden);
                        }
                        //El item existe, y posee un ID, entonces es una actualización
                        else
                        {
                            // Transición de estados
                            switch (existingObject.IdEstado)
                            {
                                case (int)EstadosOrden.Registrada:
                                    if (orden.IdPerito != 0)
                                    {
                                        //Tiene perito asignado
                                        orden.IdEstado = (int)EstadosOrden.Asignada;
                                    }
                                    else
                                    {
                                        //No tiene perito asignado
                                        orden.IdPerito = null;
                                    }
                                    break;

                                case (int)EstadosOrden.Asignada:
                                    if (orden.IdPerito == 0)
                                    {
                                        //No tiene perito asignado
                                        orden.IdPerito = null;
                                        orden.IdEstado = (int)EstadosOrden.Registrada;
                                    }
                                    break;

                                case (int)EstadosOrden.Descargada:
                                    if (orden.IdPerito == 0)
                                    {
                                        //No tiene perito asignado
                                        orden.IdPerito = null;
                                        orden.IdEstado = (int)EstadosOrden.Registrada;
                                    }
                                    else if (orden.IdPerito != existingObject.IdPerito)
                                    {
                                        //Se ha reasignado a un perito, pasa a Asignada
                                        orden.IdEstado = (int)EstadosOrden.Asignada;
                                        // avisamos al perito anterior
                                        idPeritoAnt = existingObject.IdPerito.Value;

                                    }
                                    break;
                            }

                            _DataContext.ApplyCurrentValues<OrdenTrabajo>("OrdenTrabajo", orden);
                        }

                        _DataContext.SaveChanges();

                        // Lanzamos las notificaciones correspondientes
                        // Respecto a las notificaciones push se pueden producir:
                        // 1. Inserciones de ordenes. En caso que lleven perito asignado tenemos que notificar sólo al perito asignado
                        // 2. Actualizaciones. En caso que tuviese perito asignado, y se reasigne a otro perito, hay que avisar a ambos

                        // Si ha habido una reasignación, tenemos que avisar al perito anterior
                        if (idPeritoAnt!=0)
                        {
                            // avisamos al perito anterior
                            EnviarToast(idPeritoAnt);
                        } 
                        
                        // Si la orden tiene perito asignado, hay que avisarle en cualquier caso
                        // para que actualice los datos modificados de la orden
                        if (orden.IdEstado == (int)EstadosOrden.Asignada)
                        {
                            EnviarToast(orden.IdPerito.Value);
                        }

                    }

                    return orden;
                }
                else
                {
                    // Usuario no logeado
                    return null;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Da de baja a una orden de trabajo
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Orden a dar de baja</param>
        /// <returns>Devuelve true si ha sido posible eliminar la orden de trabajo, false en caso contrario</returns>
        public bool DeleteOrden(Usuario gestor, int idOrdenTrabajo)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        OrdenTrabajo existingObject = _DataContext.OrdenTrabajo.Where(x => x.IdOrdenTrabajo == idOrdenTrabajo).FirstOrDefault();

                        //Se comprueba si el item no existe en la base de datos pero si tiene PK
                        if (existingObject == null)
                        {
                            //Error, situación inconsistente. Se ha pasado una PK que no existe
                            throw new Exception(string.Format("El idOrdenTrabajo {0} no existe.", idOrdenTrabajo));
                        }

                        //Borra el objeto con la PK idOrdenTrabajo
                        _DataContext.DeleteObject(existingObject);
                        _DataContext.SaveChanges();

                        // Lanzamos la notificación push al perito
                        if (existingObject.IdPerito != null)
                        {
                            EnviarToast((int)existingObject.IdPerito);
                        }
                    }

                    return true;
                }
                else
                {
                    // Usuario no logeado
                    return false;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Devuelve el fichero de la foto
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="idOrdenTrabajo">Identificador de la orden para recuperar la foto</param>
        /// <returns>Un array de bytes que contiene el fichero de la foto</returns>
        public byte[] GetFoto(Usuario gestor, int idOrdenTrabajo)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) != null)
                {

                    using (SGPMEntities _DataContext = new SGPMEntities())
                    {

                        Foto existingObject = _DataContext.Foto.Where(x => x.IdOrdenTrabajo == idOrdenTrabajo).FirstOrDefault();

                        //Se comprueba la existencia de foto
                        if (existingObject != null)
                        {
                            byte[] response;
                            string filePath = string.Concat(ConfigurationManager.AppSettings["PictureUploadPath"],
                                       existingObject.Ruta);
                            response = File.ReadAllBytes(filePath);
                            return response;
                        }
                    }
                }

                // No existe el usuario o no se ha encontrado ninguna imagen
                return null;
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }
        #endregion

        #region Notificaciones Toast

        /// <summary>
        /// Envia una notificación Toast al dispositivo de un perito concreto
        /// </summary>
        /// <param name="idPerito">Identificador del perito al que se desea enviar la notificación. Se utiliza para
        /// poder extraer la Uri asociada a ese perito</param>
        private void EnviarToast(int idPerito)
        {
            string subscriptionUri = string.Empty;

            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    Usuario perito = _DataContext.Usuario.Where(x => x.IdUsuario == idPerito).FirstOrDefault();

                    if (string.IsNullOrEmpty(perito.SubscriptionURI))
                    {
                        // Todavía no se dispone de URI para notificar al perito, por lo que se sale del método
                        return;
                    }

                    // Obtenemos la URI que devuelve Microsoft Push Notification Service al cliente push cuando se crea el canal de notificaciones
                    subscriptionUri = perito.SubscriptionURI;
                }

                HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(subscriptionUri);

                // Crea un HTTPWebRequest que postea la notificación toast al servicio de Microsoft.
                // HTTP POST es el único método permitido para enviar la notificación
                sendNotificationRequest.Method = "POST";

                // El mensaje toast
                string toastMessage = string.Format(
                    @"<?xml version=""1.0"" encoding=""utf-8""?>
                        <wp:Notification xmlns:wp=""WPNotification"">
                            <wp:Toast>
                                <wp:Text1>{0}</wp:Text1>
                            </wp:Toast>
                        </wp:Notification>",
                    "Hay actualizaciones en órdenes", "Hay");

                // Array de bytes con la notificación a enviar
                byte[] notificationMessage = Encoding.UTF8.GetBytes(toastMessage);

                // Configuración de los atributos del envío
                sendNotificationRequest.ContentLength = notificationMessage.Length;
                sendNotificationRequest.ContentType = "text/xml";
                sendNotificationRequest.Headers.Add("X-WindowsPhone-Target", "toast");
                sendNotificationRequest.Headers.Add("X-NotificationClass", "2");


                using (Stream requestStream = sendNotificationRequest.GetRequestStream())
                {
                    requestStream.Write(notificationMessage, 0, notificationMessage.Length);
                }

                // La notificación se envía y se recupera la respuesta
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();
                string notificationStatus = response.Headers["X-NotificationStatus"];
                string notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];

                // Display the response from the Microsoft Push Notification Service.  
                // Normally, error handling code would be here. In the real world, because data connections are not always available,
                // notifications may need to be throttled back if the device cannot be reached.
                // Tracer.Logger.Info (notificationStatus + " | " + deviceConnectionStatus + " | " + notificationChannelStatus);
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
            }

        }
        #endregion

        #region Informes
        /// <summary>
        /// Devuelve el informe de ratio de siniestralidad, por año y cliente. El ratio de siniestralidad nos informa del porcentaje
        ///	de órdenes que sobre el total tiene cada cliente.
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="anno">Año para el que solicita el listado</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los clientes que han tenido siniestros (órdenes) en ese año, indicando su ratio de siniestralidad.</returns>
        public List<vRatioSiniestralidad> ListRatioSiniestralidad(Usuario gestor, int anno,
            int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vRatioSiniestralidad> query = _DataContext.vRatioSiniestralidad;
                    // Filtros
                    query = query.Where(s => s.Anno == anno);

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("Ratio ASC"))
                    {
                        query = query.OrderBy(p => p.Ratio);
                    }
                    else if (sorting.Equals("Ratio DESC"))
                    {
                        query = query.OrderByDescending(p => p.Ratio);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.Ratio); //Por defecto!
                    }

                    // Retorna la lista de ordenes de trabajo
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve el número de filas que aparecen en un informe de ratio de siniestralidad para un año concreto
        /// </summary>
        /// <param name="anno">Año del que se desean obtener el informe</param>
        /// <returns>Un valor entero con el número de filas que aparecen en un informe de ratio de siniestralidad para un año concreto</returns>
        public int GetRatioSiniestralidadCountByFilter(int anno)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vRatioSiniestralidad> query = _DataContext.vRatioSiniestralidad;

                    // Filtros
                    query = query.Where(s => s.Anno == anno);

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }

        /// <summary>
        /// Devuelve el informe de ratio de productividad, por año y cliente. El ratio de productividad nos informa del porcentaje
        ///	de órdenes que sobre el total tiene cada perito.
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <param name="anno">Año para el que solicita el listado</param>
        /// <param name="startIndex">Índice que se emplea en la paginación. Indica a partir de qué registro se lista</param>
        /// <param name="count">Número de registros a devolver. Es el tamaño de la página</param>
        /// <param name="sorting">Columna por la que se desea ordenar</param>
        /// <returns>Un listado con todos los peritos que han gestionado siniestros (órdenes) en ese año, indicando su ratio de productividad.</returns>
        public List<vRatioProductividad> ListRatioProductividad(Usuario gestor, int anno,
            int startIndex, int count, string sorting)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vRatioProductividad> query = _DataContext.vRatioProductividad;
                    // Filtros
                    query = query.Where(s => s.Anno == anno);

                    // Ordenación
                    if (string.IsNullOrEmpty(sorting) || sorting.Equals("Ratio ASC"))
                    {
                        query = query.OrderBy(p => p.Ratio);
                    }
                    else if (sorting.Equals("Ratio DESC"))
                    {
                        query = query.OrderByDescending(p => p.Ratio);
                    }
                    else
                    {
                        query = query.OrderBy(p => p.Ratio); //Por defecto!
                    }

                    // Retorna la lista de ordenes de trabajo
                    return count > 0
                               ? query.Skip(startIndex).Take(count).ToList() //Con paginación
                               : query.ToList(); //Sin paginación
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        /// <summary>
        /// Devuelve el número de filas que aparecen en un informe de ratio de productividad para un año concreto
        /// </summary>
        /// <param name="anno">Año del que se desean obtener el informe</param>
        /// <returns>Un valor entero con el número de filas que aparecen en un informe de ratio de productividad para un año concreto</returns>
        public int GetRatioProductividadCountByFilter(int anno)
        {
            try
            {
                using (SGPMEntities _DataContext = new SGPMEntities())
                {
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    IEnumerable<vRatioProductividad> query = _DataContext.vRatioProductividad;

                    // Filtros
                    query = query.Where(s => s.Anno == anno);

                    return query.Count();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return -1;
            }
        }

        /// <summary>
        /// Devuelve un listado con los años de los que se tienen registradas órdenes de trabajo valoradas
        /// </summary>
        /// <param name="gestor">Usuario gestor que solicita la operación</param>
        /// <returns>Un listado con todos los años de los que se tienen registradas órdenes valoradas.</returns>
        public List<int?> ListAnnos(Usuario gestor)
        {
            try
            {

                // Se comprueba si el usuario está validado
                if (ValidaUsuario(gestor.Login, gestor.Password) == null)
                {
                    return null;
                }

                using (SGPMEntities _DataContext = new SGPMEntities())
                {

                    //Deshabilitamos la carga perezosa y la creación de proxy
                    _DataContext.ContextOptions.LazyLoadingEnabled = false;
                    _DataContext.ContextOptions.ProxyCreationEnabled = false;

                    // Retorna la lista de años
                    return _DataContext.vRatioSiniestralidad.Select(s => s.Anno).Distinct().ToList();
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return null;
            }
        }

        #endregion


    }
}
